@@ -1,13 +1,13 @@
 # -*- coding: utf-8 -*-
 
-"""Top-level package for sRNA analysis pipeline."""
+"""Top-level package for RNA sequencing analysis pipeline."""
 import warnings
 
 __all__ = ['general', 'filtering', 'enrichment', '__attr_file_key__', '__biotype_file_key__']
 __name__ = "rnalysis"
 __author__ = """Guy Teichman"""
 __email__ = "guyteichman@gmail.com"
-__version__ = "1.3.4"
+__version__ = "2.1.0"
 __license__ = "MIT"
 __attr_file_key__ = "attribute_reference_table"""
 __biotype_file_key__ = "biotype_reference_table"
