@@ -36,10 +36,12 @@ def load_csv(filename: str, index_col: int = None, drop_columns: Union[str, List
     assert isinstance(filename,
                       (str, Path)), f"Filename must be of type str or pathlib.Path, is instead {type(filename)}."
     encoding = 'ISO-8859-1'
-    kwargs = dict(sep=None, engine='python', encoding=encoding, squeeze=squeeze, comment=comment, skipinitialspace=True)
+    kwargs = dict(sep=None, engine='python', encoding=encoding, comment=comment, skipinitialspace=True)
     if index_col is not None:
         kwargs['index_col'] = index_col
     df = pd.read_csv(filename, **kwargs)
+    if squeeze:
+        df = df.squeeze("columns")
     df.index = [ind.strip() if isinstance(ind, str) else ind for ind in df.index]
     if isinstance(df, pd.DataFrame):
         df.columns = [col.strip() if isinstance(col, str) else col for col in df.columns]
@@ -433,7 +435,19 @@ def map_taxon_id(taxon_name: Union[str, int]) -> Tuple[int, str]:
         raise ValueError(f"No taxons match the search query '{taxon_name}'.")
     header = res[0].split('\t')
 
-    matched_taxon = res[1].split('\t')
+    if isinstance(taxon_name, int):
+        matched_taxon = None
+        for line in res[1::]:
+            split_line = line.split('\t')
+            if int(split_line[0]) == taxon_name:
+                matched_taxon = split_line
+                break
+        if matched_taxon is None:
+            matched_taxon = res[1].split('\t')
+    else:
+        best_candidate_line = np.argmin([int(line.split('\t')[0]) for line in res[1::]])
+        matched_taxon = res[best_candidate_line + 1].split('\t')
+
     taxon_id = int(matched_taxon[header.index('Taxon')])
     scientific_name = matched_taxon[header.index('Scientific name')]
     if len(res) > 2 and not (taxon_name == taxon_id or taxon_name == scientific_name):
