@@ -1,5 +1,8 @@
 import pytest
+import pandas as pd
 from collections import namedtuple
+from sklearn.preprocessing import PowerTransformer, StandardScaler
+
 import matplotlib
 from rnalysis.filtering import *
 import os
@@ -465,6 +468,40 @@ def test_intersection():
     assert set1.intersection(set2, inplace=False) == intersection_truth
 
 
+@pytest.mark.parametrize("this_set,other_sets,majority_threshold,truth",
+                         [(Filter("tests/test_files/test_deseq.csv"),
+                           [{'WBGene00000001', 'WBGene00000002', 'WBGene00000003'},
+                            {'WBGene00000002', 'WBGene00000004'}], 2 / 3,
+                           {'WBGene00000002', 'WBGene00000003', 'WBGene00000004'}),
+                          (DESeqFilter('tests/test_files/test_deseq_set_ops_1.csv'),
+                           [DESeqFilter('tests/test_files/test_deseq_set_ops_2.csv'),
+                            {'WBGene00008447', 'WBGene00021018'}], 2 / 3, {'WBGene00008447',
+                                                                           'WBGene00013816',
+                                                                           'WBGene00018199',
+                                                                           'WBGene00019174',
+                                                                           'WBGene00021018',
+                                                                           'WBGene00021019',
+                                                                           'WBGene00021375',
+                                                                           'WBGene00044258',
+                                                                           'WBGene00045366',
+                                                                           'WBGene00045410',
+                                                                           'WBGene00194708',
+                                                                           'WBGene00219304',
+                                                                           'WBGene00219307',
+                                                                           'WBGene00010100',
+                                                                           'WBGene00077437',
+                                                                           'WBGene00023036',
+                                                                           'WBGene00012648',
+                                                                           'WBGene00022486',
+                                                                           'WBGene00007674'}),
+                          (DESeqFilter('tests/test_files/test_deseq_set_ops_1.csv'),
+                           [DESeqFilter('tests/test_files/test_deseq_set_ops_2.csv'),
+                            {'WBGene00008447', 'WBGene00021018'}], 1, set())])
+def test_majority_vote_intersection(this_set, other_sets, majority_threshold, truth):
+    result = this_set.majority_vote_intersection(*other_sets, majority_threshold=majority_threshold)
+    assert result == truth
+
+
 def test_union():
     intersection_truth = {'WBGene00021375', 'WBGene00044258', 'WBGene00219304', 'WBGene00194708', 'WBGene00018199',
                           'WBGene00019174', 'WBGene00021019', 'WBGene00013816', 'WBGene00045366', 'WBGene00219307',
@@ -1466,6 +1503,17 @@ def test_split_hdbscan_api():
         c.split_hdbscan(c.shape[0] + 1)
 
 
+def test_split_clicom_api():
+    c = CountFilter('tests/test_files/big_counted.csv')
+    c.filter_top_n(by='cond1rep1', n=1000)
+    res = c.split_clicom({'method': 'hdbscan', 'min_cluster_size': [20, 75, 40], 'metric': ['ys1', 'yr1', 'spearman']},
+                         {'method': 'hierarchical', 'n_clusters': [7, 12], 'metric': ['euclidean', 'jackknife', 'yr1'],
+                          'linkage': ['average', 'ward']},
+                         {'method': 'kmedoids', 'n_clusters': [7, 16], 'metric': 'spearman'},
+                         power_transform=[False, True], evidence_threshold=0.5, min_cluster_size=5)
+    _test_correct_clustering_split(c, res, True)
+
+
 def test_split_kmedoids_api():
     c = CountFilter('tests/test_files/big_counted.csv')
     c.filter_top_n(by='cond1rep1', n=2000)
@@ -1578,3 +1626,43 @@ def test_avg_subsamples(sample_list, truth_path):
 def test_triplicates(input_file, expected_triplicates):
     counted = CountFilter(input_file)
     assert counted.triplicates == expected_triplicates
+
+
+def _log2_plus1(df: pd.DataFrame):
+    return np.log2(df + 1)
+
+
+def _log10_plus1(df: pd.DataFrame):
+    return np.log10(df + 1)
+
+
+def _box_cox_plus1(df: pd.DataFrame):
+    return PowerTransformer(method='box-cox').fit_transform(df + 1)
+
+
+def _multiply_by_3_reduce_2(df: pd.DataFrame):
+    return (df * 3) - 2
+
+
+@pytest.mark.parametrize('filter_obj,columns',
+                         [(Filter('tests/test_files/counted_6cols.csv'), 'all'),
+                          (DESeqFilter('tests/test_files/test_deseq.csv'), 'baseMean'),
+                          (CountFilter('tests/test_files/counted.csv'), ['cond1', 'cond4'])])
+@pytest.mark.parametrize('function,kwargs,matching_function',
+                         [('log2', {}, _log2_plus1),
+                          ('log10', {}, _log10_plus1),
+                          (_log2_plus1, {}, _log2_plus1),
+                          ('box-cox', {}, _box_cox_plus1),
+                          (lambda x, mult, red: (x * mult) - red, {'mult': 3, 'red': 2}, _multiply_by_3_reduce_2)])
+def test_transform(filter_obj, columns, function, kwargs, matching_function):
+    truth = filter_obj.__copy__()
+    if columns == 'all':
+        truth.df = matching_function(truth.df)
+    else:
+        if len(truth.df[columns].shape) == 1:
+            truth.df[columns] = matching_function(truth.df[columns].to_frame())
+        else:
+            truth.df[columns] = matching_function(truth.df[columns])
+    filter_obj.transform(function, columns, **kwargs)
+
+    filter_obj.df.equals(truth.df)
